#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : fpos.py
# Contents: accretion/frequent item set mining: false positives
# Author  : Christian Borgelt
# History : 2012.02.06 file created as accfim.py
#           2012.05.30 adapted to different set of experiments
#           2012.07.02 command list setup made separate function
#           2013.01.16 adapted to psts.py and fim library
#           2013.01.18 bug in command setup fixed (parameter thresh)
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, path
from os      import makedirs
from os.path import join
from time    import strftime
from random  import seed
UTILDIR  = join('..', 'util')
if UTILDIR  not in path: path.insert(0, UTILDIR)
from id2par  import run
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts, psts2tras
from fimcmds import get_cmds

#-----------------------------------------------------------------------

def fpos (ncnt=100, period=3, rate=20, width=0.003,
          target='c', runs=1000, verbose=100, srng=1):
    if srng == 0: srng = 1      # init. seed and get data file name
    name = 'n%d_p%g_r%g_w%g_%c' % (ncnt,period,rate,width*1000,target)
    cmds = get_cmds(target)     # get the different mining commands
    pcs  = dict([(c, dict()) for c in cmds])
    for i in range(runs):       # repeat the experiments multiple times
        if verbose > 0 and i % verbose == 0:
            print(strftime('%Y.%m.%d %H:%M:%S fpos ')+name+' '+str(i))
        seed(srng)              # seed random number generator
        psts = genpsts(ncnt, period, rate)
        tras = psts2tras(psts, width, period)
        for c in cmds:          # generate trains and traverse commands
            fn,args = cmds[c]   # mine the generated spike trains
            pats = set([(frozenset(p),r[0]) for p,r in fn(tras,**args)])
            cnts = pcs[c]       # get the counters for the command
            for p,r in pats:    # and traverse the found patterns
                s = (len(p),r)  # get the pattern signature
                if s in cnts: cnts[s] += 1
                else:         cnts[s]  = 1
        srng += 2               # update pattern counters and seed
    if verbose > 0:             # print termination log message
        print(strftime('%Y.%m.%d %H:%M:%S fpos ')
                      +name+' '+str(runs)+' done.')
    try: makedirs('data')       # ensure that a directory exists
    except OSError: pass        # for the result files
    n = 1.0/runs if runs > 0 else 1
    for c in cmds:              # traverse the different commands
        cnts = pcs[c]           # write pattern counters per command
        with open(join('data','fp_'+c+'_'+name+'.txt'), 'w') as out:
            for x in sorted([(s[0], s[1], cnts[s]*n) for s in cnts]):
                out.write('%g %g %g\n' % x)
    return pcs                  # return the pattern counters

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [   20 ] ),
                ( 'target', [ 'm', 'c' ] ),
                ( 'width',  [ 0.005, 0.004, 0.003 ] ),
                ( 'runs',   [ 1000 ] ) ] ]

    run(fpos, grids, int(argv[1])-1 if len(argv) > 1 else -1)
